/******************************************************************************
USB-RAD121 Example Code
    Copyright (C) 2013 Magnii Technologies, LLC
    http://www.magnii.com/
    Release Date: May 4, 2013

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
******************************************************************************/

#include "stdafx.h"
#include <windows.h>
#include <time.h>
#include "CUSB_RAD121.h"

/****************************************************
We use FTDI'S D2XX Driver DLL to access the device.
The one included in this project is for Windows, 
but you can download versions for other operating
systems (Linux, MacOS X) from FTDI's website at
http://www.ftdichip.com/Drivers/D2XX.htm
****************************************************/
#include "FTD2XX.h"

/****************************************************
CUSB_RAD121()
    Class constructor.
****************************************************/
CUSB_RAD121::CUSB_RAD121()
{
} /* CUSB_RAD121() */


/****************************************************
~CUSB_RAD121()
    Class destructor.
****************************************************/
CUSB_RAD121::~CUSB_RAD121()
{
    Close();
} /* ~CUSB_RAD121() */


/****************************************************
Open()
    Opens a connection to the USB-RAD121 device.

    Returns true on success, false otherwise.
****************************************************/
bool CUSB_RAD121::Open()
{
    // Open the device
    FT_STATUS ftStatus = FT_OpenEx("USB-RAD121 Geiger Counter", FT_OPEN_BY_DESCRIPTION, &ftHandle);

    // Change the baud rate
    ftStatus |= FT_SetBaudRate( ftHandle, 921600 );

    // Reset device counter
    ftStatus |= FT_Purge( ftHandle, FT_PURGE_RX | FT_PURGE_TX );

    // Initialize LastCountsTime
    LastCountsTime = clock();

    // Return true for success
    return( ftStatus == FT_OK );
} /* Open() */


/****************************************************
Close()
    Closes the connection.

    Returns true on success, false otherwise.
****************************************************/
bool CUSB_RAD121::Close()
{
    FT_STATUS ftStatus = FT_Close( ftHandle );

    // Return true for success
    return( ftStatus == FT_OK );
} /* Close() */


/****************************************************
GetReading()
    Get the counts per minute value since the
    last function call.  The time you wait between
    calls directly affects the resolution (the longer
    you wait, the higher the resolution of your reading).

    *CountsPerMinute gets the new reading, or -1 on error.
    Returns true on success, false otherwise.
****************************************************/
bool CUSB_RAD121::GetReading( double *CountsPerMinute )
{
    double elapsed_seconds;
    unsigned long counts;
    double counts_per_second;
    clock_t CountsTime;

    // Get the total counts from the device
    FT_STATUS ftStatus = FT_GetQueueStatus( ftHandle, &counts );

    // Get the time we read the counts at
    CountsTime = clock();

    // Reset device counter
    ftStatus |= FT_Purge( ftHandle, FT_PURGE_RX | FT_PURGE_TX );

    // Calculate the counts per minute
    elapsed_seconds = (double)(CountsTime - LastCountsTime) / CLOCKS_PER_SEC;

    // Calculate the counts per second
    if( elapsed_seconds > 0 )
    {
        counts_per_second = counts / elapsed_seconds;
    }
    else
    {
        counts_per_second = 0;
    }

    // Save the new LastCountsTime
    LastCountsTime = CountsTime;

    // Return true for success
    if( ftStatus == FT_OK )
    {
        // Convert to counts per minute and save to pointer
        *CountsPerMinute = 60.0 * counts_per_second;
        return true;
    }
    else
    {
        // Error reading device
        *CountsPerMinute = -1;
        return false;
    }
} /* GetReading() */


/****************************************************
SetDeviceBeep()
    Enables the disables the beep for each
    count on the device.

    Returns true on success, false otherwise.
****************************************************/
bool CUSB_RAD121::SetDeviceBeep(bool BeepEnabled)
{
    FT_STATUS ftStatus;

    if ( BeepEnabled )
    {
        ftStatus = FT_ClrRts(ftHandle);
    }
    else
    {
        ftStatus = FT_SetRts(ftHandle);
    }

    // Return true for success
    return( ftStatus == FT_OK );
} /* SetDeviceBeep() */


/****************************************************
Calculate_CompensatedCPM()
    CountsPerMinute: Value to calculate.

    Returns CountsPerMinute that has been compensated
    for geiger tube dead time.  This will affect high
    reading levels, but not lower level readings.
****************************************************/
double CUSB_RAD121::Calculate_CompensatedCPM( double CountsPerMinute )
{
    return CountsPerMinute / (1.0 - (CountsPerMinute/60.0) * USB_RAD121_DEAD_TIME );
} /* Calculate_CompensatedCPM() */


/****************************************************
Calculate_mRhr()
    CountsPerMinute: Value to calculate.

    Returns the CountsPerMinute value that has been
    converted to millirem per hour (mR/hr)
****************************************************/
double CUSB_RAD121::Calculate_mRhr( double CountsPerMinute )
{
    return CountsPerMinute / USB_RAD121_SENSITIVITY;
} /* Calculate_mRhr() */
